@@ -25,6 +25,7 @@
 
 #include <config.h>
 
+#include <stdio.h>
 #include <ctype.h>
 #include <string.h>
 #include <gtk/gtk.h>
@@ -162,49 +163,66 @@ enum {
 };
 
 static void
-setup_combo (GnomeDItemEdit *dee, int type, const char *extra)
+setup_combo (GnomeDItemEdit *dee,
+	     int             type,
+	     const char     *extra)
 {
 	GList *types = NULL;
 
-	if (type == ONLY_DIRECTORY) {
+	switch (type) {
+	case ONLY_DIRECTORY:
 		types = g_list_prepend (types, "Directory");
-	} else {
+		break;
+	default: 
 		types = g_list_prepend (types, "Application");
+
+		if (type != ALL_EXCEPT_DIRECTORY)
+			types = g_list_prepend (types, "Directory");
+
 		types = g_list_prepend (types, "Link");
 		types = g_list_prepend (types, "FSDevice");
 		types = g_list_prepend (types, "MimeType");
-		if (type != ALL_EXCEPT_DIRECTORY)
-			types = g_list_prepend (types, "Directory");
 		types = g_list_prepend (types, "Service");
 		types = g_list_prepend (types, "ServiceType");
+		break;
 	}
-	if (extra != NULL)
-		types = g_list_prepend (types, (char *)extra);
 
-	gtk_combo_set_popdown_strings
-		(GTK_COMBO (dee->_priv->type_combo), types);
+	if (extra != NULL)
+		types = g_list_prepend (types, (char *) extra);
+
+	g_assert (types != NULL);
+
+	gtk_combo_set_popdown_strings (
+		GTK_COMBO (dee->_priv->type_combo), types);
 
 	g_list_free (types);
 }
 
 static void 
-table_attach_entry(GtkTable * table, GtkWidget * w,
-		   gint l, gint r, gint t, gint b)
+table_attach_entry (GtkTable  *table,
+		    GtkWidget *entry,
+		    int        left,
+		    int        right,
+		    int        top,
+		    int        bottom)
 {
-        gtk_table_attach(table, w, l, r, t, b,
-                         GTK_EXPAND | GTK_FILL | GTK_SHRINK,
-                         GTK_FILL,
-                         0, 0);
+        gtk_table_attach (
+		table, entry, left, right, top, bottom,
+		GTK_EXPAND | GTK_FILL | GTK_SHRINK, GTK_FILL,
+		0, 0);
 }
 
 static void
-table_attach_label(GtkTable * table, GtkWidget * w,
-		   gint l, gint r, gint t, gint b)
+table_attach_label (GtkTable  *table,
+		    GtkWidget *label,
+		    int        left,
+		    int        right,
+		    int        top,
+		    int        bottom)
 {
-        gtk_table_attach(table, w, l, r, t, b,
-                         GTK_FILL,
-                         GTK_FILL,
-                         0, 0);
+        gtk_table_attach(
+		table, label, left, right, top, bottom,
+		GTK_FILL, GTK_FILL, 0, 0);
 }
 
 static void 
@@ -243,101 +261,101 @@ type_combo_changed (GnomeDItemEdit *dee)
 }
 
 static void
-fill_easy_page(GnomeDItemEdit * dee, GtkWidget * table)
+fill_easy_page (GnomeDItemEdit *dee,
+		GtkWidget      *table)
 {
-        GtkWidget *label;
-        GtkWidget *hbox;
-        GtkWidget *align;
+	GtkWidget *label;
+	GtkWidget *entry;
+	GtkWidget *hbox;
+	GtkWidget *align;
+	GtkWidget *combo;
+	GtkWidget *icon_entry;
+	GtkWidget *check_button;
 
-        label = label_new(_("Name:"));
-        table_attach_label(GTK_TABLE(table), label, 0, 1, 0, 1);
+	label = label_new (_("Name:"));
+	table_attach_label (GTK_TABLE (table), label, 0, 1, 0, 1);
 
-        dee->_priv->name_entry = gtk_entry_new();
-        table_attach_entry(GTK_TABLE(table),dee->_priv->name_entry, 1, 2, 0, 1);
+	entry = gtk_entry_new ();
+	table_attach_entry (GTK_TABLE (table), entry, 1, 2, 0, 1);
 
-        g_signal_connect_object (G_OBJECT(dee->_priv->name_entry),
-                                 "changed",
-                                 G_CALLBACK(gnome_ditem_edit_changed),
-                                 G_OBJECT(dee),
-                                 G_CONNECT_SWAPPED);		
-        g_signal_connect_object (G_OBJECT(dee->_priv->name_entry),
-                                 "changed",
-                                 G_CALLBACK(gnome_ditem_edit_name_changed),
-                                 G_OBJECT(dee),
-                                 G_CONNECT_SWAPPED);
+	g_signal_connect_object (entry, "changed",
+				 G_CALLBACK (gnome_ditem_edit_changed),
+				 dee, G_CONNECT_SWAPPED);		
 
-        label = label_new(_("Comment:"));
-        table_attach_label(GTK_TABLE(table),label, 0, 1, 1, 2);
+	g_signal_connect_object (entry, "changed",
+				 G_CALLBACK (gnome_ditem_edit_name_changed),
+				 dee, G_CONNECT_SWAPPED);
+	dee->_priv->name_entry = entry;
 
-        dee->_priv->comment_entry = gtk_entry_new();
-        table_attach_entry(GTK_TABLE(table),dee->_priv->comment_entry, 1, 2, 1, 2);
+	label = label_new (_("Comment:"));
+	table_attach_label (GTK_TABLE (table), label, 0, 1, 1, 2);
 
-        g_signal_connect_object (G_OBJECT(dee->_priv->comment_entry),		
-                                 "changed",
-                                 G_CALLBACK(gnome_ditem_edit_changed),
-                                 G_OBJECT(dee),
-                                 G_CONNECT_SWAPPED);
-        dee->_priv->exec_label = label= label_new (_("Command:"));
-        table_attach_label (GTK_TABLE (table), label, 0, 1, 2, 3);
+	entry = gtk_entry_new ();
+	table_attach_entry (GTK_TABLE (table), entry, 1, 2, 1, 2);
 
-        dee->_priv->exec_entry = gtk_entry_new();
-        table_attach_entry(GTK_TABLE(table),dee->_priv->exec_entry, 1, 2, 2, 3);
+	g_signal_connect_object (entry, "changed",
+				 G_CALLBACK (gnome_ditem_edit_changed),
+                                 dee, G_CONNECT_SWAPPED);
+	dee->_priv->comment_entry = entry;
+
+	label = label_new (_("Command:"));
+	table_attach_label (GTK_TABLE (table), label, 0, 1, 2, 3);
+	dee->_priv->exec_label = label;
+
+	entry = gtk_entry_new ();
+	table_attach_entry (GTK_TABLE (table), entry, 1, 2, 2, 3);
  
-        g_signal_connect_object (G_OBJECT(dee->_priv->exec_entry),
-                                 "changed",
-                                 G_CALLBACK (gnome_ditem_edit_changed),
-                                 G_OBJECT(dee),
-                                 G_CONNECT_SWAPPED);
-        dee->_priv->type_label = label = label_new(_("Type:"));
-        table_attach_label(GTK_TABLE(table), label, 0, 1, 3, 4);
+	g_signal_connect_object (entry, "changed",
+				 G_CALLBACK (gnome_ditem_edit_changed),
+				 dee, G_CONNECT_SWAPPED);
+	dee->_priv->exec_entry = entry;
 
-        dee->_priv->type_combo = gtk_combo_new();
+	label = label_new (_("Type:"));
+	table_attach_label (GTK_TABLE (table), label, 0, 1, 3, 4);
+	dee->_priv->type_label = label;
+
+	dee->_priv->type_combo = combo = gtk_combo_new ();
 	setup_combo (dee, ALL_TYPES, NULL);
-        gtk_combo_set_value_in_list(GTK_COMBO(dee->_priv->type_combo), 
-                                    FALSE, TRUE);
-        table_attach_entry(GTK_TABLE(table),dee->_priv->type_combo, 1, 2, 3, 4);
 
-        g_signal_connect_object (G_OBJECT(GTK_COMBO(dee->_priv->type_combo)->entry),
-                                 "changed",
-                                 G_CALLBACK (gnome_ditem_edit_changed),
-                                 G_OBJECT(dee),
-                                 G_CONNECT_SWAPPED);	
-        g_signal_connect_object (G_OBJECT(GTK_COMBO(dee->_priv->type_combo)->entry),
-                                 "changed",
-                                 G_CALLBACK (type_combo_changed),
-                                 G_OBJECT(dee),
-                                 G_CONNECT_SWAPPED);
+	gtk_combo_set_value_in_list (GTK_COMBO (combo), FALSE, TRUE);
+	table_attach_entry (GTK_TABLE (table), combo, 1, 2, 3, 4);
 
+	g_signal_connect_object (GTK_COMBO (combo)->entry, "changed",
+				 G_CALLBACK (gnome_ditem_edit_changed),
+				 dee, G_CONNECT_SWAPPED);	
+	g_signal_connect_object (GTK_COMBO (combo)->entry, "changed",
+				 G_CALLBACK (type_combo_changed),
+				 dee, G_CONNECT_SWAPPED);
+	dee->_priv->type_combo = combo;
 
-        label = label_new(_("Icon:"));
-        table_attach_label(GTK_TABLE(table), label, 0, 1, 4, 5);
+	label = label_new (_("Icon:"));
+	table_attach_label (GTK_TABLE (table), label, 0, 1, 4, 5);
 
-        hbox = gtk_hbox_new(FALSE, GNOME_PAD_BIG);
-        gtk_table_attach(GTK_TABLE(table), hbox,
-                         1, 2, 4, 5,
-                         GTK_EXPAND | GTK_FILL,
-                         0,
-                         0, 0);
+	hbox = gtk_hbox_new (FALSE, GNOME_PAD_BIG);
+	gtk_table_attach (GTK_TABLE (table), hbox, 1, 2, 4, 5,
+			  GTK_EXPAND | GTK_FILL, 0, 0, 0); 
 
-        dee->_priv->icon_entry = gnome_icon_entry_new ("desktop-icon", _("Browse icons"));
-        g_signal_connect_swapped(G_OBJECT(dee->_priv->icon_entry), "changed",
-                                 G_CALLBACK (gnome_ditem_edit_changed),
-                                 G_OBJECT(dee));
-        g_signal_connect_swapped(G_OBJECT(dee->_priv->icon_entry), "changed",
-                                 G_CALLBACK (gnome_ditem_edit_icon_changed),
-                                 G_OBJECT(dee));
-        gtk_box_pack_start (GTK_BOX (hbox), dee->_priv->icon_entry,
-                                 FALSE, FALSE, 0);
+        icon_entry = gnome_icon_entry_new (
+			"desktop-icon", _("Browse icons"));
 
-        align = gtk_alignment_new (0.0, 0.5, 0.0, 0.0);
-        gtk_box_pack_start (GTK_BOX (hbox), align, FALSE, FALSE, 0);
+        g_signal_connect_swapped (icon_entry, "changed",
+				  G_CALLBACK (gnome_ditem_edit_changed), dee);
+        g_signal_connect_swapped (icon_entry, "changed",
+                                  G_CALLBACK (gnome_ditem_edit_icon_changed), dee);
+        gtk_box_pack_start (
+		GTK_BOX (hbox), icon_entry, FALSE, FALSE, 0);
+        dee->_priv->icon_entry = icon_entry;
+
+	align = gtk_alignment_new (0.0, 0.5, 0.0, 0.0);
+	gtk_box_pack_start (GTK_BOX (hbox), align, FALSE, FALSE, 0);
+
+        check_button = gtk_check_button_new_with_label (_("Run in Terminal"));
+        g_signal_connect_swapped (check_button, "clicked",
+				  G_CALLBACK (gnome_ditem_edit_changed), dee);
+
+        gtk_container_add (GTK_CONTAINER (align), check_button);
+        dee->_priv->terminal_button = check_button;
 
-        dee->_priv->terminal_button = gtk_check_button_new_with_label (_("Run in Terminal"));
-        g_signal_connect_swapped(G_OBJECT(dee->_priv->terminal_button), 
-                                 "clicked",
-                                 G_CALLBACK (gnome_ditem_edit_changed),
-                                 G_OBJECT(dee));
-        gtk_container_add (GTK_CONTAINER (align),dee->_priv->terminal_button);
 }
 
 static GtkTreeIter*
@@ -377,256 +395,257 @@ return_iter_nth_row(GtkTreeView  *tree_view,
 }
 
 static void
-set_iter_nth_row (GtkTreeView *tree_view,
-                  GtkTreeIter *iter,
-                  gint        row)
+translations_select_row (GtkTreeSelection *selection,
+			 GnomeDItemEdit   *dee)
 {
-        GtkTreeModel *tree_model;
+        GtkTreeModel *model = NULL;
+        GtkTreeIter   iter;
+        char         *lang;
+        char         *name;
+        char         *comment;
 
-        tree_model = gtk_tree_view_get_model (tree_view);
-        gtk_tree_model_get_iter_root (tree_model, iter);
-        iter = return_iter_nth_row (tree_view, tree_model, iter, 0 , row);
-}
+	 if (!gtk_tree_selection_get_selected (selection, &model, &iter))
+		return;
 
-static void
-translations_select_row(GtkTreeView *cl, int row, int column,
-			GdkEvent *event, GnomeDItemEdit *dee)
-{
-        char *lang;
-        char *name;
-        char *comment;
-        GtkTreeIter iter;
-        GtkTreeModel *model;
-        GValue value = {0, };	
+	gtk_tree_model_get (
+		model, &iter, 0, &lang, 1, &name, 2, &comment, -1);
 
-        model = gtk_tree_view_get_model (GTK_TREE_VIEW (cl));	
-        set_iter_nth_row (cl,&iter,row);
-
-        gtk_tree_model_get_value (model,&iter,0,&value);
-        lang = g_strdup (g_value_get_string (&value));
-
-        gtk_tree_model_get_value (model,&iter,1, &value);
-        name = g_strdup (g_value_get_string (&value));
-
-        gtk_tree_model_get_value (model,&iter,2, &value);
-        comment = g_strdup (g_value_get_string (&value));
-        g_value_unset (&value);
-
-        gtk_entry_set_text(GTK_ENTRY(dee->_priv->transl_lang_entry), lang);
-        gtk_entry_set_text(GTK_ENTRY(dee->_priv->transl_name_entry), name);
-        gtk_entry_set_text(GTK_ENTRY(dee->_priv->transl_comment_entry), comment);
+        gtk_entry_set_text(
+		GTK_ENTRY (dee->_priv->transl_lang_entry), lang);
+        gtk_entry_set_text (
+		GTK_ENTRY (dee->_priv->transl_name_entry), name);
+        gtk_entry_set_text(
+		GTK_ENTRY (dee->_priv->transl_comment_entry), comment);
 	
         g_free (lang);
         g_free (comment);
         g_free (name);
 }
 
-int 
-count_rows (GtkTreeView *view)
-{
-        int rows = 0;
-        GtkTreeModel *model;
-        GtkTreeIter iter;
-
-        model = gtk_tree_view_get_model (view);
-        gtk_tree_model_get_iter_root (model, &iter);
-
-        while (gtk_tree_model_iter_next (model, &iter))
-             rows++;
-
-        return rows;
-}
-
 static void
 translations_add (GtkWidget      *button,
 		  GnomeDItemEdit *dee)
 {
-        int i = 0;
-        int number_of_rows = 0;
-        const char *lang;
-        const char *name;
-        const char *comment;
-        const char *text[3];
-        const GList *language_list;
-        const char *curlang;
-        GtkTreeView *tree;
-        GtkTreeIter iter;
-        GtkTreeModel *model;
+	GtkTreeView  *tree;
+	GtkTreeModel *model;
+	GtkTreeIter   iter;
+	const GList  *langs;
+	const char   *tmp;
+	const char   *name;
+	const char   *comment;
+	char         *lang;
+	gboolean      ret;
 
-        tree = GTK_TREE_VIEW (dee->_priv->translations);
-        model = gtk_tree_view_get_model (tree);
+	tmp     = gtk_entry_get_text (GTK_ENTRY (dee->_priv->transl_lang_entry));
+	name    = gtk_entry_get_text (GTK_ENTRY (dee->_priv->transl_name_entry));
+	comment = gtk_entry_get_text (GTK_ENTRY (dee->_priv->transl_comment_entry));
 
-        lang     = gtk_entry_get_text (GTK_ENTRY (dee->_priv->transl_lang_entry));
-        name    = gtk_entry_get_text (GTK_ENTRY (dee->_priv->transl_name_entry));
-        comment = gtk_entry_get_text (GTK_ENTRY (dee->_priv->transl_comment_entry));
-  
-        g_assert (lang != NULL && name != NULL && comment != NULL);
+	g_assert (tmp != NULL && name != NULL && comment != NULL);
 	
-        lang = g_strstrip (g_strdup (lang));
-	
-        /*we are setting the current language so set the easy page entries*/
-        /*FIXME: do the opposite as well!, but that's not that crucial*/
-        language_list = gnome_i18n_get_language_list("LC_ALL");
-        curlang = language_list ? language_list->data : NULL;
-        if ((curlang && strcmp(curlang,lang)==0) ||
-            ((!curlang || strcmp(curlang,"C")==0) && !*lang)) {
-                gtk_entry_set_text(GTK_ENTRY(dee->_priv->name_entry),name);
-                gtk_entry_set_text(GTK_ENTRY(dee->_priv->comment_entry),comment);
-        }
-        gtk_tree_model_get_iter_root (model, &iter);
-        number_of_rows = count_rows (tree);
-        for (i=0;i <number_of_rows;i++){
-                char *s;
-                GValue value = {0, };
-                gtk_tree_model_get_value (model,&iter,0, &value);
-                s = g_strdup (g_value_get_string (&value));
-                g_value_unset (&value);
+	lang = g_strstrip (g_strdup (tmp));
 
-                if (!strcmp (lang, s)) {
-                        gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-					    1, name,2, comment, -1);
-                        g_signal_emit (dee, ditem_edit_signals [CHANGED], 0);
-                        g_free (s);
-                        return;
-                }
+	if (!lang [0]) {
+		g_free (lang);
+		return;
+	}
 
-                gtk_tree_model_iter_next (model, &iter);
-                g_free (s);
-        }
-        text[0]=lang;
-        text[1]=name;
-        text[2]=comment;
-        gtk_list_store_append (GTK_LIST_STORE(model), &iter);
-        gtk_list_store_set (GTK_LIST_STORE(model),&iter, 0, text[0],1,text[1],2,text[2],-1);
-        g_signal_emit(G_OBJECT(dee), ditem_edit_signals[CHANGED], 0);
+	/*
+	 * If we are editing the current language, change the name and
+	 * comment entries on the easy page as well.
+	 */
+	langs = gnome_i18n_get_language_list ("LC_ALL");
+	tmp = langs ? langs->data : NULL;
+	if ((tmp && !strcmp (tmp, lang)) || (!tmp && !strcmp (tmp, "C"))) {
+		gtk_entry_set_text (
+			GTK_ENTRY (dee->_priv->name_entry), name);
+		gtk_entry_set_text (
+			GTK_ENTRY (dee->_priv->comment_entry), comment);
+	}
+
+	tree  = GTK_TREE_VIEW (dee->_priv->translations);
+	model = gtk_tree_view_get_model (tree);
+
+	ret = gtk_tree_model_get_iter_first (model, &iter);
+        while (ret) {
+		char *string;
+
+		gtk_tree_model_get (model, &iter, 0, &string, -1);
+
+		if (!strcmp (lang, string)) {
+			gtk_list_store_set (
+				GTK_LIST_STORE (model), &iter,
+				1, name, 2, comment, -1);
+
+			g_signal_emit (dee, ditem_edit_signals [CHANGED], 0);
+
+			g_free (string);
+			g_free (lang);
+
+			return;
+		}
+
+		g_free (string);
+
+		ret = gtk_tree_model_iter_next (model, &iter);
+	}
+
+	gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+	gtk_list_store_set (
+		GTK_LIST_STORE (model), &iter,
+		0, lang, 1, name, 2, comment, -1);
+
+	gtk_editable_delete_text (
+		GTK_EDITABLE (dee->_priv->transl_lang_entry), 0, -1);
+	gtk_editable_delete_text (
+		GTK_EDITABLE (dee->_priv->transl_name_entry), 0, -1);
+	gtk_editable_delete_text (
+		GTK_EDITABLE (dee->_priv->transl_comment_entry), 0, -1);
+
+	g_signal_emit (dee, ditem_edit_signals [CHANGED], 0);
+
+	g_free (lang);
 }
 
 static void
 translations_remove (GtkWidget      *button,
 		     GnomeDItemEdit *dee)
 {
-        GtkTreeView *view;
+        GtkTreeView      *view;
         GtkTreeSelection *selection;
-        GtkTreeModel *model;
-        GtkTreeIter iter;
+        GtkTreeModel     *model;
+        GtkTreeIter       iter;
 
         view = GTK_TREE_VIEW (dee->_priv->translations);
         selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
 
-        /* gtk_tree_selection_get_selected will return selected node if
-	 * selection is set to GTK_SELECTION_SINGLE
-	 */
-
-        /* just return if nothing selected */
         if (!gtk_tree_selection_get_selected (selection, &model, &iter))
                return;
 
         gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
-        g_signal_emit(G_OBJECT(dee), ditem_edit_signals[CHANGED], 0);
+
+        g_signal_emit (G_OBJECT(dee), ditem_edit_signals [CHANGED], 0);
+}
+
+static GtkWidget *
+setup_translations_list (GnomeDItemEdit *dee)
+{
+	GtkCellRenderer   *renderer;
+	GtkTreeViewColumn *column;
+	GtkTreeSelection  *selection;
+	GtkListStore      *model;
+	GtkWidget         *tree;
+
+	model = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+	tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
+	g_object_unref (model);
+
+	renderer = gtk_cell_renderer_text_new ();	
+
+	column = gtk_tree_view_column_new_with_attributes (
+					_("Language"), renderer,
+					"text", 0, NULL);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
+
+	column = gtk_tree_view_column_new_with_attributes (
+					_("Name"), renderer,
+					"text", 1, NULL);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
+
+	column = gtk_tree_view_column_new_with_attributes (
+					_("Comment"), renderer,
+					"text", 2, NULL);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (tree), column);
+
+	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (tree));
+
+	gtk_tree_view_set_headers_clickable (GTK_TREE_VIEW (tree), FALSE);
+
+        selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
+
+	g_signal_connect (selection, "changed",
+			  G_CALLBACK (translations_select_row), dee);
+
+	return tree;
 }
  
 static void
 fill_advanced_page (GnomeDItemEdit *dee,
 		    GtkWidget      *page)
 {
-        GtkWidget         *label;
-        GtkWidget         *button;
-        GtkWidget         *box;
-        const char        *transl [3];
-        GtkCellRenderer   *renderer;
-        GtkTreeViewColumn *column;
-	GtkListStore      *model;
+	GtkWidget *label;
+	GtkWidget *entry;
+	GtkWidget *button;
+	GtkWidget *box;
 
-        dee->_priv->tryexec_label = label = label_new(_("Try this before using:"));
-        table_attach_label(GTK_TABLE(page),label, 0, 1, 0, 1);
+	label = label_new (_("Try this before using:"));
+	table_attach_label (GTK_TABLE (page), label, 0, 1, 0, 1);
+	dee->_priv->tryexec_label = label;
 
-        dee->_priv->tryexec_entry = gtk_entry_new();
-        table_attach_entry(GTK_TABLE(page),dee->_priv->tryexec_entry, 1, 2, 0, 1);
-        g_signal_connect_swapped(G_OBJECT(dee->_priv->tryexec_entry), 
-                                  "changed",
-                                  G_CALLBACK(gnome_ditem_edit_changed),
-                                  G_OBJECT(dee));
+	entry = gtk_entry_new ();
+	table_attach_entry (GTK_TABLE (page), entry, 1, 2, 0, 1);
+	g_signal_connect_swapped (entry, "changed",
+				  G_CALLBACK (gnome_ditem_edit_changed), dee);
+	dee->_priv->tryexec_entry = entry;
 
-        label = label_new(_("Documentation:"));
-        table_attach_label(GTK_TABLE(page),label, 0, 1, 1, 2);
+	label = label_new (_("Documentation:"));
+	table_attach_label (GTK_TABLE (page), label, 0, 1, 1, 2);
 
-        dee->_priv->doc_entry = gtk_entry_new();
-        gtk_entry_set_max_length (GTK_ENTRY(dee->_priv->doc_entry), 255);        
-        table_attach_entry(GTK_TABLE(page),dee->_priv->doc_entry, 1, 2, 1, 2);
-        g_signal_connect_swapped(G_OBJECT(dee->_priv->doc_entry), 
-                                  "changed",
-                                  G_CALLBACK(gnome_ditem_edit_changed),
-                                  G_OBJECT(dee));
+	entry = gtk_entry_new ();
+	gtk_entry_set_max_length (GTK_ENTRY (entry), 255);        
+	table_attach_entry (GTK_TABLE (page), entry, 1, 2, 1, 2);
+	g_signal_connect_swapped (entry, "changed",
+				  G_CALLBACK (gnome_ditem_edit_changed), dee);
+	dee->_priv->doc_entry = entry;
 
-        label = gtk_label_new(_("Name/Comment translations:"));
-        table_attach_label(GTK_TABLE(page),label, 0, 2, 2, 3);
+	label = gtk_label_new (_("Name/Comment translations:"));
+	table_attach_label (GTK_TABLE (page), label, 0, 2, 2, 3);
+
+	box = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
+	table_attach_entry (GTK_TABLE (page), box, 0, 2, 3, 4);
+
+	entry = gtk_entry_new ();
+	gtk_box_pack_start (GTK_BOX (box), entry, TRUE, TRUE, 0);
+	gtk_widget_set_size_request (entry, 30, -1);
+	dee->_priv->transl_lang_entry = entry;
+
+	entry = gtk_entry_new ();
+	gtk_box_pack_start (GTK_BOX (box), entry, TRUE, TRUE, 0);
+	dee->_priv->transl_name_entry = entry;
+
+	entry = gtk_entry_new ();
+	gtk_box_pack_start (GTK_BOX (box), entry, TRUE, TRUE, 0);
+	dee->_priv->transl_comment_entry = entry;
+
+	box = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
+	table_attach_entry (GTK_TABLE (page), box, 0, 2, 4, 5);
   
-        transl [0] = _("Language");
-        transl [1] = _("Name");
-        transl [2] = _("Comment");
+	button = gtk_button_new_with_label (_("Add/Set"));
+	gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);
+	g_signal_connect (button, "clicked",
+			  G_CALLBACK (translations_add), dee);
 
-	model = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
-        dee->_priv->translations =
-		gtk_tree_view_new_with_model (GTK_TREE_MODEL (model));
-	g_object_unref (model);
-
-        renderer = gtk_cell_renderer_text_new ();	
-
-        column = gtk_tree_view_column_new_with_attributes (transl [0], renderer, NULL);
-        gtk_tree_view_append_column (GTK_TREE_VIEW (dee->_priv->translations), column);
-
-        column = gtk_tree_view_column_new_with_attributes (transl [1], renderer, NULL);
-        gtk_tree_view_append_column (GTK_TREE_VIEW (dee->_priv->translations), column);
-
-        column = gtk_tree_view_column_new_with_attributes (transl [2], renderer, NULL);
-        gtk_tree_view_append_column (GTK_TREE_VIEW (dee->_priv->translations), column);
-
-        gtk_tree_view_columns_autosize (GTK_TREE_VIEW (dee->_priv->translations));
-
-        box = gtk_scrolled_window_new (NULL, NULL);
-        gtk_widget_set_size_request (box, 0, 120);
-        gtk_container_add (GTK_CONTAINER (box),dee->_priv->translations);
-        table_attach_list (GTK_TABLE (page), box, 0, 2, 5, 6);
-
-        gtk_tree_view_set_headers_clickable (GTK_TREE_VIEW (dee->_priv->translations),FALSE);
-        g_signal_connect (dee->_priv->translations, "select_row",
-			  G_CALLBACK (translations_select_row), dee);
-
-        box = gtk_hbox_new(FALSE,GNOME_PAD_SMALL);
-        table_attach_entry(GTK_TABLE(page),box, 0, 2, 3, 4);
+	button = gtk_button_new_with_label (_("Remove"));
+	gtk_box_pack_start (GTK_BOX (box), button, FALSE, FALSE, 0);
+	g_signal_connect (button, "clicked",
+			  G_CALLBACK (translations_remove), dee);
   
-        dee->_priv->transl_lang_entry = gtk_entry_new();
-        gtk_box_pack_start(GTK_BOX(box),dee->_priv->transl_lang_entry,FALSE,FALSE,0);
-        gtk_widget_set_size_request(dee->_priv->transl_lang_entry,50,0);
+	dee->_priv->translations = setup_translations_list (dee);
 
-        dee->_priv->transl_name_entry = gtk_entry_new();
-        gtk_box_pack_start(GTK_BOX(box),dee->_priv->transl_name_entry,TRUE,TRUE,0);
-
-        dee->_priv->transl_comment_entry = gtk_entry_new();
-        gtk_box_pack_start(GTK_BOX(box),dee->_priv->transl_comment_entry,TRUE,TRUE,0);
-
-        box = gtk_hbox_new(FALSE,GNOME_PAD_SMALL);
-        table_attach_entry(GTK_TABLE(page),box, 0, 2, 4, 5);
-  
-        button = gtk_button_new_with_label(_("Add/Set"));
-        gtk_box_pack_start(GTK_BOX(box),button,FALSE,FALSE,0);
-        g_signal_connect(G_OBJECT(button),"clicked",
-                           G_CALLBACK(translations_add),
-                           dee);
-        button = gtk_button_new_with_label(_("Remove"));
-        gtk_box_pack_start(GTK_BOX(box),button,FALSE,FALSE,0);
-        g_signal_connect(G_OBJECT(button),"clicked",
-                           G_CALLBACK(translations_remove),
-                           dee);
+	box = gtk_scrolled_window_new (NULL, NULL);
+	gtk_widget_set_size_request (box, 0, 120);
+	gtk_container_add (GTK_CONTAINER (box), dee->_priv->translations);
+	table_attach_list (GTK_TABLE (page), box, 0, 2, 5, 6);
 }
 
 static GtkWidget *
 make_page (void)
 {
-        GtkWidget * frame, * page;
+        GtkWidget *frame;
+	GtkWidget *page;
 
         frame = gtk_frame_new (NULL);
-        gtk_container_set_border_width (GTK_CONTAINER(frame), GNOME_PAD_SMALL);
+        gtk_container_set_border_width (
+			GTK_CONTAINER (frame), GNOME_PAD_SMALL);
 
         page = gtk_table_new (5, 2, FALSE);
         gtk_container_set_border_width (GTK_CONTAINER (page), GNOME_PAD_SMALL);
@@ -641,32 +660,31 @@ make_page (void)
 static void
 gnome_ditem_edit_instance_init (GnomeDItemEdit *dee)
 {
-	GtkWidget *child1, *child2;
-	dee->_priv = g_new0(GnomeDItemEditPrivate, 1);
+	GtkWidget *page;
 
-        child1 = make_page();
-        fill_easy_page (dee, GTK_BIN (child1)->child);
-        gtk_widget_show_all (child1);
+	dee->_priv = g_new0 (GnomeDItemEditPrivate, 1);
 
-        child2 = make_page();
-        fill_advanced_page (dee, GTK_BIN(child2)->child);
-        gtk_widget_show_all (child2);
+        page = make_page ();
+        fill_easy_page (dee, GTK_BIN (page)->child);
+        gtk_widget_show_all (page);
 
-        gtk_notebook_append_page (GTK_NOTEBOOK (dee), 
-                                  child1,
+        gtk_notebook_append_page (GTK_NOTEBOOK (dee), page,
 				  gtk_label_new (_("Basic")));
 
-        gtk_notebook_append_page (GTK_NOTEBOOK (dee), 
-                                  child2,
+	dee->_priv->child1 = page;
+
+        page = make_page ();
+        fill_advanced_page (dee, GTK_BIN (page)->child);
+        gtk_widget_show_all (page);
+
+        gtk_notebook_append_page (GTK_NOTEBOOK (dee), page,
 				  gtk_label_new (_("Advanced")));
 
-	dee->_priv->child1 = child1;
-	dee->_priv->child2 = child2;
+	dee->_priv->child2 = page;
 
 	/* FIXME: There needs to be a way to edit ALL keys/sections */
 }
 
-
 /**
  * gnome_ditem_edit_new
  *
@@ -799,7 +817,7 @@ gnome_ditem_edit_sync_display (GnomeDItemEdit *dee)
                            cs ? cs : "");
 
         tmpstr = gnome_desktop_item_get_icon (ditem);
-	gnome_icon_entry_set_filename (GNOME_ICON_ENTRY (dee->_priv->icon_entry), cs);
+	gnome_icon_entry_set_filename (GNOME_ICON_ENTRY (dee->_priv->icon_entry), tmpstr);
 	g_free (tmpstr);
 
         cs = gnome_desktop_item_get_string (ditem, "DocPath"); /* FIXME check name */
@@ -847,121 +865,117 @@ gnome_ditem_edit_sync_display (GnomeDItemEdit *dee)
 static const char *
 get_language (void)
 {
-	const GList *list = gnome_i18n_get_language_list ("LC_MESSAGES");
-	while (list != NULL) {
-		const char *lang = list->data;
-		/* find first without encoding */
-		if (strchr (lang, '.') == NULL)
-			return lang;
-	}
+	const GList *list;
+	const GList *l;
+
+	list = gnome_i18n_get_language_list ("LC_MESSAGES");
+
+	for (l = list; l; l = l->next)
+		if (!strchr (l->data, '.'))
+			return (char *) l->data;
+
 	return NULL;
 }
 
-/* Conform ditem to display */
 static void
 gnome_ditem_edit_sync_ditem (GnomeDItemEdit *dee)
 {
-        const gchar * text;
-        gchar *file;
 	GnomeDesktopItem *ditem;
-        int i,number_of_rows = 0; 
-        GtkTreeView *view;
+	GtkTreeModel     *model;
+	GtkTreeIter       iter;
+	const char       *type;
+	const char       *uri;
+	const char       *attr;
+	char             *file;
+	gboolean          ret;
 
-        GtkTreeModel *model;
-        GtkTreeIter iter;
+	g_return_if_fail (dee != NULL);
+	g_return_if_fail (GNOME_IS_DITEM_EDIT (dee));
 
-        g_return_if_fail (dee != NULL);
-        g_return_if_fail (GNOME_IS_DITEM_EDIT(dee));
+	if (!dee->_priv->ditem)
+		dee->_priv->ditem = gnome_desktop_item_new ();
 
 	ditem = dee->_priv->ditem;
 
-	if (ditem == NULL) {
-		ditem = gnome_desktop_item_new ();
-		dee->_priv->ditem = ditem;
-	}
+	uri = gtk_entry_get_text (GTK_ENTRY (dee->_priv->exec_entry));
 
-        text = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(dee->_priv->type_combo)->entry));
-        gnome_desktop_item_set_string (ditem, GNOME_DESKTOP_ITEM_TYPE, text);
+	type = gtk_entry_get_text (
+			GTK_ENTRY (GTK_COMBO (dee->_priv->type_combo)->entry));
+	gnome_desktop_item_set_string (
+			ditem, GNOME_DESKTOP_ITEM_TYPE, type);
 
 	/* hack really */
-	if (text != NULL &&
-	    strcmp (text, "Link") == 0) {
-		text = gtk_entry_get_text(GTK_ENTRY(dee->_priv->exec_entry));
-		gnome_desktop_item_set_string (ditem, GNOME_DESKTOP_ITEM_URL, text);
-	} else {
-		text = gtk_entry_get_text(GTK_ENTRY(dee->_priv->exec_entry));
-		gnome_desktop_item_set_string (ditem, GNOME_DESKTOP_ITEM_EXEC, text);
-	}
+	if (type && !strcmp (type, "Link"))
+		attr = GNOME_DESKTOP_ITEM_URL;
+	else
+		attr  = GNOME_DESKTOP_ITEM_EXEC;
 
-        text = gtk_entry_get_text(GTK_ENTRY(dee->_priv->tryexec_entry));
-        gnome_desktop_item_set_string (ditem, GNOME_DESKTOP_ITEM_TRY_EXEC, text);
+	gnome_desktop_item_set_string (ditem, attr, uri);
+
+	gnome_desktop_item_set_string (
+		ditem, GNOME_DESKTOP_ITEM_TRY_EXEC,
+		gtk_entry_get_text (GTK_ENTRY (dee->_priv->tryexec_entry)));
   
-	file = gnome_icon_entry_get_filename (GNOME_ICON_ENTRY (dee->_priv->icon_entry));
-        gnome_desktop_item_set_string (ditem, GNOME_DESKTOP_ITEM_ICON, file);
+	file = gnome_icon_entry_get_filename (
+			GNOME_ICON_ENTRY (dee->_priv->icon_entry));
+	gnome_desktop_item_set_string (
+			ditem, GNOME_DESKTOP_ITEM_ICON, file);
 	g_free (file);
 
-        text = gtk_entry_get_text (GTK_ENTRY (dee->_priv->doc_entry));
-        gnome_desktop_item_set_string (ditem, "DocPath", text);
+	gnome_desktop_item_set_string (
+			ditem, GNOME_DESKTOP_ITEM_DOC_PATH,
+			gtk_entry_get_text (GTK_ENTRY (dee->_priv->doc_entry)));
 
-        if (GTK_TOGGLE_BUTTON (dee->_priv->terminal_button)->active)
-		gnome_desktop_item_set_boolean (ditem,
-						GNOME_DESKTOP_ITEM_TERMINAL,
-						TRUE);
-        else
-		gnome_desktop_item_set_boolean (ditem,
-						GNOME_DESKTOP_ITEM_TERMINAL,
-						FALSE);
+	gnome_desktop_item_set_boolean (
+			ditem, GNOME_DESKTOP_ITEM_TERMINAL,
+			GTK_TOGGLE_BUTTON (dee->_priv->terminal_button)->active);
 
-	gnome_desktop_item_clear_localestring (ditem,
-					       GNOME_DESKTOP_ITEM_NAME);
-	gnome_desktop_item_clear_localestring (ditem,
-					       GNOME_DESKTOP_ITEM_COMMENT);
-        view = GTK_TREE_VIEW (dee->_priv->translations);
-        number_of_rows = count_rows (view);	
-        model = gtk_tree_view_get_model (GTK_TREE_VIEW (dee->_priv->translations));
-        gtk_tree_model_get_iter_root (model, &iter);
-        for (i = 0; i < number_of_rows; i++) {
-                char *lang, *name, *comment;
-                GValue value = {0, };
+	gnome_desktop_item_clear_localestring (
+			ditem, GNOME_DESKTOP_ITEM_NAME);
+	gnome_desktop_item_clear_localestring (
+			ditem, GNOME_DESKTOP_ITEM_COMMENT);
 
-                gtk_tree_model_get_value (model,&iter,0, &value);
-                lang = g_strdup (g_value_get_string (&value));
+	model = gtk_tree_view_get_model (
+			GTK_TREE_VIEW (dee->_priv->translations));
 
-                gtk_tree_model_get_value (model,&iter,1, &value); 
-                name = g_strdup (g_value_get_string (&value));;
+	ret = gtk_tree_model_get_iter_first (model, &iter);
+	while (ret) {
+		char *lang;
+		char *name;
+		char *comment;
 
-                gtk_tree_model_get_value (model,&iter,2, &value);
-                comment = g_strdup (g_value_get_string (&value));;
-                g_value_unset (&value);
+		gtk_tree_model_get (
+			model, &iter, 0, &lang, 1, &name, 2, &comment, -1);
 
-                if(!name && !comment){
-                        g_free (lang);
-                        continue;
-                }
-                if (lang == NULL)
-                        lang = g_strdup(get_language());
+		if (!name && !comment) {
+			g_free (lang);
+			ret = gtk_tree_model_iter_next (model, &iter);
+			continue;
+		}
 
-                g_assert (lang != NULL);
-                
-                gnome_desktop_item_set_localestring_lang
-			(ditem, GNOME_DESKTOP_ITEM_NAME, lang, name);
-                gnome_desktop_item_set_localestring_lang
-			(ditem, GNOME_DESKTOP_ITEM_NAME, lang, comment);
+		if (!lang)
+			lang = g_strdup (get_language ());
 
-                g_free (name);
-                g_free (comment);
-                g_free (lang);
-                gtk_tree_model_iter_next (model, &iter);
-        }
+		gnome_desktop_item_set_localestring_lang (
+			ditem, GNOME_DESKTOP_ITEM_NAME, lang, name);
+		gnome_desktop_item_set_localestring_lang (
+			ditem, GNOME_DESKTOP_ITEM_COMMENT, lang, comment);
+		
+		g_free (name);
+		g_free (comment);
+		g_free (lang);
 
-	/* This always overrides the above in case of conflict */
-        text = gtk_entry_get_text(GTK_ENTRY(dee->_priv->name_entry));
-        gnome_desktop_item_set_localestring (ditem, GNOME_DESKTOP_ITEM_NAME, text);
+		ret = gtk_tree_model_iter_next (model, &iter);
+	}
 
-        text = gtk_entry_get_text(GTK_ENTRY(dee->_priv->comment_entry));
-        gnome_desktop_item_set_localestring (ditem, GNOME_DESKTOP_ITEM_COMMENT, text);
+	gnome_desktop_item_set_localestring (
+			ditem, GNOME_DESKTOP_ITEM_NAME,
+			gtk_entry_get_text (GTK_ENTRY(dee->_priv->name_entry)));
 
-	/* ui not dirty any more, we just synced it */
+	gnome_desktop_item_set_localestring (
+			ditem, GNOME_DESKTOP_ITEM_COMMENT,
+			gtk_entry_get_text (GTK_ENTRY(dee->_priv->comment_entry)));
+			
 	dee->_priv->ui_dirty = FALSE;
 }
 
@@ -988,16 +1002,18 @@ gnome_ditem_edit_load_uri (GnomeDItemEdit *dee,
 
         newentry = gnome_desktop_item_new_from_uri (uri, 0, error);
 
-        if (newentry != NULL) {
-		if (dee->_priv->ditem != NULL)
-			gnome_desktop_item_unref (dee->_priv->ditem);
-		dee->_priv->ditem = newentry;
-		dee->_priv->ui_dirty = TRUE;
-                gnome_ditem_edit_sync_display (dee);
-		return TRUE;
-        } else {
-                return FALSE;
-        }
+        if (!newentry)
+		return FALSE;
+
+	if (dee->_priv->ditem)
+		gnome_desktop_item_unref (dee->_priv->ditem);
+
+	dee->_priv->ditem = newentry;
+	dee->_priv->ui_dirty = TRUE;
+
+	gnome_ditem_edit_sync_display (dee);
+
+	return TRUE;
 }
 
 /**
@@ -1020,10 +1036,12 @@ gnome_ditem_edit_set_ditem (GnomeDItemEdit *dee,
         g_return_if_fail (GNOME_IS_DITEM_EDIT (dee));
         g_return_if_fail (ditem != NULL);
 
-	if (dee->_priv->ditem != NULL)
+	if (dee->_priv->ditem)
 		gnome_desktop_item_unref (dee->_priv->ditem);
+
 	dee->_priv->ditem = gnome_desktop_item_copy (ditem);
 	dee->_priv->ui_dirty = TRUE;
+
         gnome_ditem_edit_sync_display (dee);
 }
 
@@ -1044,12 +1062,14 @@ gnome_ditem_edit_get_ditem (GnomeDItemEdit *dee)
         g_return_val_if_fail (dee != NULL, NULL);
         g_return_val_if_fail (GNOME_IS_DITEM_EDIT (dee), NULL);
 
-	if (dee->_priv->ditem == NULL) {
+	if (!dee->_priv->ditem) {
 		dee->_priv->ditem = gnome_desktop_item_new ();
 		dee->_priv->ui_dirty = TRUE;
 	}
+
 	if (dee->_priv->ui_dirty)
 		gnome_ditem_edit_sync_ditem (dee);
+
 	return dee->_priv->ditem;
 }
 
@@ -1219,93 +1239,3 @@ gnome_ditem_edit_set_directory_only (GnomeDItemEdit *dee,
 		}
 	}
 }
-
-
-#ifdef TEST_DITEM_EDIT
-
-#include <libgnomeui.h>
-
-static void
-changed_callback(GnomeDItemEdit *dee, gpointer data)
-{
-        g_print("Changed!\n");
-        fflush(stdout);
-}
-
-static void
-icon_changed_callback(GnomeDItemEdit *dee, gpointer data)
-{
-        g_print("Icon changed!\n");
-        fflush(stdout);
-}
-
-static void
-name_changed_callback(GnomeDItemEdit *dee, gpointer data)
-{
-        g_print("Name changed!\n");
-        fflush(stdout);
-}
-
-int
-main(int argc, char * argv[])
-{
-        GtkWidget * app;
-        GtkWidget * notebook;
-        GtkObject * dee;
-
-        argp_program_version = VERSION;
-
-        gnome_init ("testing ditem edit", NULL, argc, argv, 0, 0);
-
-        app = bonobo_window_new("testing ditem edit", "Testing");
-
-        notebook = gtk_notebook_new();
-
-        bonobo_window_set_contents(BONOBO_WINDOW(app), notebook);
-
-        dee = gnome_ditem_edit_new(GTK_NOTEBOOK(notebook));
-
-        gnome_ditem_edit_load_file(GNOME_DITEM_EDIT(dee),
-                                   "/usr/local/share/gnome/apps/grun.desktop");
-
-#ifdef GNOME_ENABLE_DEBUG
-        g_print("Dialog (main): %p\n", GNOME_DITEM_EDIT(dee)->icon_dialog);
-#endif
-
-        g_signal_connect_swapped(G_OBJECT(app), "delete_event", 
-                                 G_CALLBACK(gtk_widget_destroy),
-                                 (app));
-
-        g_signal_connect(G_OBJECT(app), "destroy",
-                         G_CALLBACK(gtk_main_quit),
-                         NULL);
-
-        g_signal_connect (G_OBJECT((dee), "changed",
-                          G_CALLBACK(changed_callback),
-                          NULL);
-
-        g_signal_connect(G_OBJECT(dee), "icon_changed",
-                         G_CALLBACK(icon_changed_callback),
-                         NULL);
-
-        g_signal_connect(G_OBJECT(dee), "name_changed",
-                         G_CALLBACK(name_changed_callback),
-                         NULL);
-
-#ifdef GNOME_ENABLE_DEBUG
-        g_print("Dialog (main 2): %p\n", GNOME_DITEM_EDIT(dee)->icon_dialog);
-#endif
-
-        gtk_widget_show(notebook);
-        gtk_widget_show(app);
-
-#ifdef GNOME_ENABLE_DEBUG
-        g_print("Dialog (main 3): %p\n", GNOME_DITEM_EDIT(dee)->icon_dialog);
-#endif
-
-        gtk_main();
-
-        return 0;
-}
-
-#endif
